// LogScrollView.cpp  Nur fr Logische Twips
#include "stdafx.h"
#include <afxpriv.h>
#include "LogScrollView.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CLogScrollView

BEGIN_MESSAGE_MAP(CLogScrollView, CView)
	//{{AFX_MSG_MAP(CLogScrollView)
	ON_WM_SIZE()
	ON_WM_HSCROLL()
	ON_WM_VSCROLL()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CLogScrollView Konstruktion/Destruktion

CLogScrollView::CLogScrollView()
{
	// Alles mit Null initialisieren
	AFX_ZERO_INIT_OBJECT(CView);
}

CLogScrollView::~CLogScrollView()
{
}

/////////////////////////////////////////////////////////////////////////////
// CLogScrollView Zeichenoperationen

void CLogScrollView::OnPrepareDC(CDC* pDC, CPrintInfo* pInfo)
{
	ASSERT_VALID(pDC);
	if(pDC->IsPrinting()) {
		pDC->SetMapMode(MM_TWIPS);
	}
	else {
		ASSERT(m_totalDev.cx >= 0 && m_totalDev.cy >= 0);
		pDC->SetMapMode(MM_ANISOTROPIC);
		pDC->SetWindowExt(1440, 1440);
		pDC->SetViewportExt(pDC->GetDeviceCaps(LOGPIXELSX),
						-pDC->GetDeviceCaps(LOGPIXELSX));
		CPoint ptVpOrg(0, 0);       // Keine Verschiebung fr Ausdruck
		ASSERT(pDC->GetWindowOrg() == CPoint(0,0));

		// Ansichtsursprung in entgegengesetzter Richtung von Bildlauf verschieben
		ptVpOrg = -GetDeviceScrollPosition();

		if (m_bCenter) {
			CRect rect;
			GetClientRect(&rect);

			// Falls Client-Bereich grer als Gesamtgre des Ausgabegerts ist,
			// Ursprung so verschieben, dass die Ausgabe im Fenster zentriert wird.
			if (m_totalDev.cx < rect.Width())
				ptVpOrg.x = (rect.Width() - m_totalDev.cx) / 2;
			if (m_totalDev.cy < rect.Height())
				ptVpOrg.y = (rect.Height() - m_totalDev.cy) / 2;
		}
		pDC->SetViewportOrg(ptVpOrg);
	}
	CView::OnPrepareDC(pDC, pInfo);     // Standarddruckerausgabe
}

/////////////////////////////////////////////////////////////////////////////
// Modus- und Skalierungs-/Bildlaufgren einstellen
void CLogScrollView::SetLogScrollSizes(SIZE sizeTotal)
{
	ASSERT(sizeTotal.cx >= 0 && sizeTotal.cy >= 0);
    CSize sizePage(sizeTotal.cx / 2,
                   sizeTotal.cy / 2);   // seitenweiser Bildlauf
    CSize sizeLine(sizeTotal.cx / 100,
                   sizeTotal.cy / 100); // zeilenweiser Bildlauf
	m_totalLog = sizeTotal;

	//BLOCK: logische Koordinaten in Gertekoordinaten umwandeln
	{
		CWindowDC dc(NULL);
		dc.SetMapMode(MM_ANISOTROPIC);
		dc.SetWindowExt(1440, 1440);
		dc.SetViewportExt(dc.GetDeviceCaps(LOGPIXELSX),
						-dc.GetDeviceCaps(LOGPIXELSX));

		// Gesamtgre
		m_totalDev = m_totalLog;
		dc.LPtoDP((LPPOINT)&m_totalDev);
		m_pageDev = sizePage;
		dc.LPtoDP((LPPOINT)&m_pageDev);
		m_lineDev = sizeLine;
		dc.LPtoDP((LPPOINT)&m_lineDev);
		if (m_totalDev.cy < 0)
			m_totalDev.cy = -m_totalDev.cy;
		if (m_pageDev.cy < 0)
			m_pageDev.cy = -m_pageDev.cy;
		if (m_lineDev.cy < 0)
			m_lineDev.cy = -m_lineDev.cy;
	} // hier DC freigeben

	// Gertespezifische Gren anpassen
	ASSERT(m_totalDev.cx >= 0 && m_totalDev.cy >= 0);
	if (m_pageDev.cx == 0)
		m_pageDev.cx = m_totalDev.cx / 10;
	if (m_pageDev.cy == 0)
		m_pageDev.cy = m_totalDev.cy / 10;
	if (m_lineDev.cx == 0)
		m_lineDev.cx = m_pageDev.cx / 10;
	if (m_lineDev.cy == 0)
		m_lineDev.cy = m_pageDev.cy / 10;

	if (m_hWnd != NULL)
	{
		// Fenster wurde erstellt, jetzt invalidieren
		UpdateBars();
		Invalidate(TRUE);
	}
}

/////////////////////////////////////////////////////////////////////////////
// Daten einlesen

CPoint CLogScrollView::GetScrollPosition() const   // logische Koordinaten
{
	CPoint pt = GetDeviceScrollPosition();
	// pt kann negativ sein, falls m_bCenter gesetzt ist

	CWindowDC dc(NULL);
	dc.SetMapMode(MM_ANISOTROPIC);
	dc.SetWindowExt(1440, 1440);
	dc.SetViewportExt(dc.GetDeviceCaps(LOGPIXELSX),
					-dc.GetDeviceCaps(LOGPIXELSX));
	dc.DPtoLP((LPPOINT)&pt);
	return pt;
}

void CLogScrollView::ScrollToPosition(POINT pt)    // logische Koordinaten
{
	CWindowDC dc(NULL);
	dc.SetMapMode(MM_ANISOTROPIC);
	dc.SetWindowExt(1440, 1440);
	dc.SetViewportExt(dc.GetDeviceCaps(LOGPIXELSX),
					-dc.GetDeviceCaps(LOGPIXELSX));
	dc.LPtoDP((LPPOINT)&pt);

	// In Gertekoordinaten umwandeln - begrenzen, falls auerhalb des Bereichs
	int xMax = GetScrollLimit(SB_HORZ);
	int yMax = GetScrollLimit(SB_VERT);
	if (pt.x < 0)
		pt.x = 0;
	else if (pt.x > xMax)
		pt.x = xMax;
	if (pt.y < 0)
		pt.y = 0;
	else if (pt.y > yMax)
		pt.y = yMax;

	ScrollToDevicePosition(pt);
}

CPoint CLogScrollView::GetDeviceScrollPosition() const
{
	CPoint pt(GetScrollPos(SB_HORZ), GetScrollPos(SB_VERT));
	ASSERT(pt.x >= 0 && pt.y >= 0);

	if (m_bCenter)
	{
		CRect rect;
		GetClientRect(&rect);

		// Falls Client-Bereich grer als Gesamtgre des Ausgabegerts ist,
		// Ursprung so verschieben, dass die Ausgabe im Fenster zentriert wird.
		// GetDeviceScrollPosition() muss dies widerspiegeln

		if (m_totalDev.cx < rect.Width())
			pt.x = -((rect.Width() - m_totalDev.cx) / 2);
		if (m_totalDev.cy < rect.Height())
			pt.y = -((rect.Height() - m_totalDev.cy) / 2);
	}

	return pt;
}

void CLogScrollView::ScrollToDevicePosition(POINT ptDev)
{
	ASSERT(ptDev.x >= 0);
	ASSERT(ptDev.y >= 0);

	// Hinweis: ScrollToDevicePosition wird in der Klasse CLogScrollView
	// verwendet, um auerhalb des sichtbaren Bereichs befindliche Bereiche 
	// zu verschieben, insbesondere in der Seitenansicht. 
	// Da OnScrollBy die Gltigkeit des Bereichs sicherstellt, 
	// verwendet ScrollToDevicePosition hier OnScrollBy nicht.

	int xOrig = GetScrollPos(SB_HORZ);
	SetScrollPos(SB_HORZ, ptDev.x);
	int yOrig = GetScrollPos(SB_VERT);
	SetScrollPos(SB_VERT, ptDev.y);
	ScrollWindow(xOrig - ptDev.x, yOrig - ptDev.y);
}

/////////////////////////////////////////////////////////////////////////////
// Andere Hilfsfunktionen

void CLogScrollView::FillOutsideRect(CDC* pDC, CBrush* pBrush)
{
	ASSERT_VALID(pDC);
	ASSERT_VALID(pBrush);

	// Rechteck auerhalb des Bildes fllen
	CRect rect;
	GetClientRect(rect);
	ASSERT(rect.left == 0 && rect.top == 0);
	rect.left = m_totalDev.cx;
	if (!rect.IsRectEmpty())
		pDC->FillRect(rect, pBrush);    // vertikaler Streifen am seitlichen Rand
	rect.left = 0;
	rect.right = m_totalDev.cx;
	rect.top = m_totalDev.cy;
	if (!rect.IsRectEmpty())
		pDC->FillRect(rect, pBrush);    // horizontaler Streifen am unteren Rand
}

void CLogScrollView::ResizeParentToFit(BOOL bShrinkOnly)
{
	// bergeordnetes Rechteck anpassen, damit Client-Rechteck die richtige Gre hat.
	// Aktuelle Gre des Client-Bereichs wird so berechnet, als seien keine
	// Bildlaufleisten vorhanden.
	CRect rectClient;
	GetWindowRect(rectClient);
	CRect rect = rectClient;
	CalcWindowRect(rect);
	rectClient.left += rectClient.left - rect.left;
	rectClient.top += rectClient.top - rect.top;
	rectClient.right -= rect.right - rectClient.right;
	rectClient.bottom -= rect.bottom - rectClient.bottom;
	rectClient.OffsetRect(-rectClient.left, -rectClient.top);
	ASSERT(rectClient.left == 0 && rectClient.top == 0);

	// Gewnschte Gre der Ansicht festlegen
	CRect rectView(0, 0, m_totalDev.cx, m_totalDev.cy);
	if (bShrinkOnly)
	{
		if (rectClient.right <= m_totalDev.cx)
			rectView.right = rectClient.right;
		if (rectClient.bottom <= m_totalDev.cy)
			rectView.bottom = rectClient.bottom;
	}
	CalcWindowRect(rectView, CWnd::adjustOutside);
	rectView.OffsetRect(-rectView.left, -rectView.top);
	ASSERT(rectView.left == 0 && rectView.top == 0);
	if (bShrinkOnly)
	{
		if (rectClient.right <= m_totalDev.cx)
			rectView.right = rectClient.right;
		if (rectClient.bottom <= m_totalDev.cy)
			rectView.bottom = rectClient.bottom;
	}

	// Rahmengre anhand der Ansichtsgre ermitteln und festlegen
	CRect rectFrame;
	CFrameWnd* pFrame = GetParentFrame();
	ASSERT_VALID(pFrame);
	pFrame->GetWindowRect(rectFrame);
	CSize size = rectFrame.Size();
	size.cx += rectView.right - rectClient.right;
	size.cy += rectView.bottom - rectClient.bottom;
	pFrame->SetWindowPos(NULL, 0, 0, size.cx, size.cy,
		SWP_NOMOVE|SWP_NOZORDER|SWP_NOACTIVATE);
}

/////////////////////////////////////////////////////////////////////////////

void CLogScrollView::OnSize(UINT nType, int cx, int cy)
{
	CView::OnSize(nType, cx, cy);
	UpdateBars();
}

/////////////////////////////////////////////////////////////////////////////
// Hilfsfunktionen fr den Bildlauf

void CLogScrollView::CenterOnPoint(CPoint ptCenter) // In Gertekoordinaten zentrieren
{
	CRect rect;
	GetClientRect(&rect);           // Gre des Client-Fensters ermitteln

	int xDesired = ptCenter.x - rect.Width() / 2;
	int yDesired = ptCenter.y - rect.Height() / 2;

	DWORD dwStyle = GetStyle();

	if ((dwStyle & WS_HSCROLL) == 0 || xDesired < 0)
	{
		xDesired = 0;
	}
	else
	{
		int xMax = GetScrollLimit(SB_HORZ);
		if (xDesired > xMax)
			xDesired = xMax;
	}

	if ((dwStyle & WS_VSCROLL) == 0 || yDesired < 0)
	{
		yDesired = 0;
	}
	else
	{
		int yMax = GetScrollLimit(SB_VERT);
		if (yDesired > yMax)
			yDesired = yMax;
	}

	ASSERT(xDesired >= 0);
	ASSERT(yDesired >= 0);

	int xOrig = GetScrollPos(SB_HORZ);
	SetScrollPos(SB_HORZ, xDesired);
	int yOrig = GetScrollPos(SB_VERT);
	SetScrollPos(SB_VERT, yDesired);
}

/////////////////////////////////////////////////////////////////////////////
// Bildlaufleisten und CWnd-Verhalten einbinden
void CLogScrollView::GetScrollBarSizes(CSize& sizeSb)
{
	sizeSb.cx = sizeSb.cy = 0;
	DWORD dwStyle = GetStyle();

	if (GetScrollBarCtrl(SB_VERT) == NULL)
	{
		// vertikale Bildlaufleisten beeinflussen den Client-Bereich dieses Fensters
		sizeSb.cx = GetSystemMetrics(SM_CXVSCROLL);
		if (dwStyle & WS_BORDER)
			sizeSb.cx -= CX_BORDER;
	}
	if (GetScrollBarCtrl(SB_HORZ) == NULL)
	{
		// horizontgale Bildlaufleisten beeinflussen den Client-Bereich dieses Fensters
		sizeSb.cy = GetSystemMetrics(SM_CYVSCROLL);
		if (dwStyle & WS_BORDER)
			sizeSb.cy -= CY_BORDER;
	}
}

BOOL CLogScrollView::GetTrueClientSize(CSize& size, CSize& sizeSb)
	// TRUE zurckgeben, falls gengend Raum fr Bildlaufleisten verfgbar ist
{
	CRect rect;
	GetClientRect(&rect);
	ASSERT(rect.top == 0 && rect.left == 0);
	size.cx = rect.right;
	size.cy = rect.bottom;
	DWORD dwStyle = GetStyle();

	// Gre der Bildlaufleisten dieses Fensters ermitteln
	GetScrollBarSizes(sizeSb);

	// Zuerst die Gre einer potentiellen Bildlaufleiste berechnen
	// (Bildlaufleistensteuerelemente werden nicht de/aktiviert)
	if (sizeSb.cx != 0 && (dwStyle & WS_VSCROLL))
	{
		// vertikale Bildlaufleisten beeinflussen den Client-Bereich dieses Fensters
		size.cx += sizeSb.cx;   // gegenwrtig aktiviert - jetzt anpassen
	}
	if (sizeSb.cy != 0 && (dwStyle & WS_HSCROLL))
	{
		// horizontale  Bildlaufleisten beeinflussen den Client-Bereich dieses Fensters
		size.cy += sizeSb.cy;   //  gegenwrtig aktiviert - jetzt anpassen
	}

	// TRUE zurckgeben, falls gengend Raum vorhanden ist
	return (size.cx > sizeSb.cx && size.cy > sizeSb.cy);
}

//  Hilfsfunktion, die Status der Bildlaufleisten zurckgibt, ohne
//  deren Status zu ndern.
void CLogScrollView::GetScrollBarState(CSize sizeClient, CSize& needSb,
	CSize& sizeRange, CPoint& ptMove, BOOL bInsideClient)
{
	// Gre der Bildlaufleisten ermitteln (den Teil, der sich im Client-Bereich befindet)
	CSize sizeSb;
	GetScrollBarSizes(sizeSb);

	// Gengend Raum zum Hinzufgen von Bildlaufleisten
	sizeRange = m_totalDev - sizeClient;
		// > 0 => Bildlauf erforderlich
	ptMove = GetDeviceScrollPosition();
		// anzusteuernden Punkt (beginndend bei gegenwrtiger Position)

	BOOL bNeedH = sizeRange.cx > 0;
	if (!bNeedH)
		ptMove.x = 0;                       // zurck zum Ursprung
	else if (bInsideClient)
		sizeRange.cy += sizeSb.cy;          // Raum fr Bildlaufleiste

	BOOL bNeedV = sizeRange.cy > 0;
	if (!bNeedV)
		ptMove.y = 0;                       // zurck zum Ursprung
	else if (bInsideClient)
		sizeRange.cx += sizeSb.cx;          // Raum fr Bildlaufleiste

	if (bNeedV && !bNeedH && sizeRange.cx > 0)
	{
		ASSERT(bInsideClient);
		// Es wird eine horizontale Bildlaufleiste bentigt.
		bNeedH = TRUE;
		sizeRange.cy += sizeSb.cy;
	}

	// Bildlauf durchfhren, falls aktuelle Position ber Grenzwert verschoben wird
	if (sizeRange.cx > 0 && ptMove.x >= sizeRange.cx)
		ptMove.x = sizeRange.cx;
	if (sizeRange.cy > 0 && ptMove.y >= sizeRange.cy)
		ptMove.y = sizeRange.cy;

	// Bildlaufleisten nach Bedarf aktualisieren
	needSb.cx = bNeedH;
	needSb.cy = bNeedV;

	// needSb, sizeRange und ptMove sind nun aktualisiert
}

void CLogScrollView::UpdateBars()
{
	// UpdateBars kann bewirken, dass die Fenstergre  verndert wird,
	// diese Grennderungen ignorieren
	if (m_bInsideUpdate)
		return;         // Rekursive Aufrufe nicht zulassen

	// Rekursion unterbinden
	m_bInsideUpdate = TRUE;

	// Horizontale Mae aktualisieren
	// Hinweis: Das De-/Aktivieren der Bildlaufleisten bewirkt 'OnSize'-Callbacks
	ASSERT(m_totalDev.cx >= 0 && m_totalDev.cy >= 0);

	CRect rectClient;
	BOOL bCalcClient = TRUE;

	// bergeordnetes Fenster darf zuerst sein Layout festlegen
	CWnd* pParentWnd = GetParent();
	if (pParentWnd != NULL)
	{
		// Falls das bergeordnete Fenster auf diese Nachricht antwortet, 
		// nur Client-Bereich fr Bildlaufleistenberechnungen verwenden
		if ((BOOL)pParentWnd->SendMessage(WM_RECALCPARENT, 0,
			(LPARAM)(LPCRECT)&rectClient) != 0)
		{
			// Zur Berechnung der Gre des Client-Bereichs
			// rectClient statt GetTrueClientSize verwenden.
			bCalcClient = FALSE;
		}
	}

	CSize sizeClient;
	CSize sizeSb;

	if (bCalcClient)
	{
		// Client-Rechteck ermitteln
		if (!GetTrueClientSize(sizeClient, sizeSb))
		{
			// Nicht gengend Raum fr Bildlaufleisten (blich bei Elementen der Gre 0)
			CRect rect;
			GetClientRect(&rect);
			if (rect.right > 0 && rect.bottom > 0)
			{
				// Falls der gesamte Client-Bereich nicht sichtbar ist, gehen wir davon
				// aus, dass wir die Bildlaufleisten steuern knnen
				//  control over our scrollbars
				EnableScrollBarCtrl(SB_BOTH, FALSE);
			}
			m_bInsideUpdate = FALSE;
			return;
		}
	}
	else
	{
		// bergeordnetes Fenster soll das "Client-"Rechteck festlegen
		GetScrollBarSizes(sizeSb);
		sizeClient.cx = rectClient.right - rectClient.left;
		sizeClient.cy = rectClient.bottom - rectClient.top;
	}

	// Gengend Raum zum Hinzufgen von Bildlaufleisten
	CSize sizeRange;
	CPoint ptMove;
	CSize needSb;

	// Anhand des echten Client-Bereichs den Status der Bildlaufleiste ermitteln
	GetScrollBarState(sizeClient, needSb, sizeRange, ptMove, bCalcClient);
	if (needSb.cx)
		sizeClient.cy -= sizeSb.cy;
	if (needSb.cy)
		sizeClient.cx -= sizeSb.cx;

	// Fenster erst nach Bedarf verschieben,
	ScrollToDevicePosition(ptMove); // damit wird auch die Position der Bildlaufleisten festgelegt

	// Diese Struktur wird zur Aktualisierung des Seitenbereichs der Bildlaufleisten bentigt
	SCROLLINFO info;
	info.fMask = SIF_PAGE|SIF_RANGE;
	info.nMin = 0;

	// Bildlaufleisten aktualisieren
	EnableScrollBarCtrl(SB_HORZ, needSb.cx);
	if (needSb.cx)
	{
		info.nPage = sizeClient.cx;
		info.nMax = m_totalDev.cx-1;
		if (!SetScrollInfo(SB_HORZ, &info, TRUE))
			SetScrollRange(SB_HORZ, 0, sizeRange.cx, TRUE);
	}
	EnableScrollBarCtrl(SB_VERT, needSb.cy);
	if (needSb.cy)
	{
		info.nPage = sizeClient.cy;
		info.nMax = m_totalDev.cy-1;
		if (!SetScrollInfo(SB_VERT, &info, TRUE))
			SetScrollRange(SB_VERT, 0, sizeRange.cy, TRUE);
	}

	// Rekursionssperre entfernen
	m_bInsideUpdate = FALSE;
}

void CLogScrollView::CalcWindowRect(LPRECT lpClientRect, UINT nAdjustType)
{
	if (nAdjustType == adjustOutside)
	{
		// Speziellen Client-Randtyp zulassen
		::AdjustWindowRectEx(lpClientRect, 0, FALSE, GetExStyle());

		// Falls die Ansicht einbettet wird, Gre der Bildlaufleisten ermitteln
		// (Bildlaufleisten sollten auerhalb direkt aktivierter Ansichten angezeigt werden)
		CSize sizeClient(
			lpClientRect->right - lpClientRect->left,
			lpClientRect->bottom - lpClientRect->top);

		CSize sizeRange = m_totalDev - sizeClient;
			// > 0 => Bildlauf erforderlich

		// Gre der Bildlaufleisten ermitteln (wird zur Anpassung der Fenstergre verwendet)
		CSize sizeSb;
		GetScrollBarSizes(sizeSb);

		// Fenstergre gem dem Status aktualisieren
		if (sizeRange.cy > 0)
		{   // vertikale Bildlaufleisten nehmen auf der x-Achse Raum ein
			lpClientRect->right += sizeSb.cx;
		}
		if (sizeRange.cx > 0)
		{   // vertikale Bildlaufleisten nehmen auf der y-Achse Raum ein
			lpClientRect->bottom += sizeSb.cy;
		}
	}
	else
	{
		// call default to handle other non-client areas
		::AdjustWindowRectEx(lpClientRect, GetStyle(), FALSE,
			GetExStyle() & ~(WS_EX_CLIENTEDGE));
	}
}

/////////////////////////////////////////////////////////////////////////////
// CLogScrollView Bildlaufoperationen

void CLogScrollView::OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar)
{
	if (pScrollBar != NULL && pScrollBar->SendChildNotifyLastMsg())
		return;     // eat it

	// Bildlaufnachrichten von anderen Steuerelementen ignorieren 
	if (pScrollBar != GetScrollBarCtrl(SB_HORZ))
		return;

	OnScroll(MAKEWORD(nSBCode, -1), nPos);
}

void CLogScrollView::OnVScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar)
{
	if (pScrollBar != NULL && pScrollBar->SendChildNotifyLastMsg())
		return;     

	// Bildlaufnachrichten von anderen Steuerelementen ignorieren 
	if (pScrollBar != GetScrollBarCtrl(SB_VERT))
		return;

	OnScroll(MAKEWORD(-1, nSBCode), nPos);
}

BOOL CLogScrollView::OnScroll(UINT nScrollCode, UINT nPos, BOOL bDoScroll)
{
	// Neue x-Position berechnen
	int x = GetScrollPos(SB_HORZ);
	int xOrig = x;

	switch (LOBYTE(nScrollCode))
	{
	case SB_TOP:
		x = 0;
		break;
	case SB_BOTTOM:
		x = INT_MAX;
		break;
	case SB_LINEUP:
		x -= m_lineDev.cx;
		break;
	case SB_LINEDOWN:
		x += m_lineDev.cx;
		break;
	case SB_PAGEUP:
		x -= m_pageDev.cx;
		break;
	case SB_PAGEDOWN:
		x += m_pageDev.cx;
		break;
	case SB_THUMBTRACK:
		x = nPos;
		break;
	}

	// Neue y-Position berechnen
	int y = GetScrollPos(SB_VERT);
	int yOrig = y;

	switch (HIBYTE(nScrollCode))
	{
	case SB_TOP:
		y = 0;
		break;
	case SB_BOTTOM:
		y = INT_MAX;
		break;
	case SB_LINEUP:
		y -= m_lineDev.cy;
		break;
	case SB_LINEDOWN:
		y += m_lineDev.cy;
		break;
	case SB_PAGEUP:
		y -= m_pageDev.cy;
		break;
	case SB_PAGEDOWN:
		y += m_pageDev.cy;
		break;
	case SB_THUMBTRACK:
		y = nPos;
		break;
	}

	BOOL bResult = OnScrollBy(CSize(x - xOrig, y - yOrig), bDoScroll);
	if (bResult && bDoScroll)
		UpdateWindow();

	return bResult;
}

BOOL CLogScrollView::OnScrollBy(CSize sizeScroll, BOOL bDoScroll)
{
	int xOrig, x;
	int yOrig, y;

	// Keinen Bildlauf durchfhren, falls kein gltiger Bildlaufbereich 
	// vorhanden ist (d.h. keine Bildlaufleiste)
	CScrollBar* pBar;
	DWORD dwStyle = GetStyle();
	pBar = GetScrollBarCtrl(SB_VERT);
	if ((pBar != NULL && !pBar->IsWindowEnabled()) ||
		(pBar == NULL && !(dwStyle & WS_VSCROLL)))
	{
		// vertikale Bildlaufleiste nicht aktiviert
		sizeScroll.cy = 0;
	}
	pBar = GetScrollBarCtrl(SB_HORZ);
	if ((pBar != NULL && !pBar->IsWindowEnabled()) ||
		(pBar == NULL && !(dwStyle & WS_HSCROLL)))
	{
		// horizontale Bildlaufleiste nicht aktiviert
		sizeScroll.cx = 0;
	}

	// Aktuelle x-Position anpassen
	xOrig = x = GetScrollPos(SB_HORZ);
	int xMax = GetScrollLimit(SB_HORZ);
	x += sizeScroll.cx;
	if (x < 0)
		x = 0;
	else if (x > xMax)
		x = xMax;

	// Aktuelle y-Position anpassen
	yOrig = y = GetScrollPos(SB_VERT);
	int yMax = GetScrollLimit(SB_VERT);
	y += sizeScroll.cy;
	if (y < 0)
		y = 0;
	else if (y > yMax)
		y = yMax;

	// Hat sich etwas gendert?
	if (x == xOrig && y == yOrig)
		return FALSE;

	if (bDoScroll)
	{
		// Bildlauf durchfhren und Positionen aktualisieren
		ScrollWindow(-(x-xOrig), -(y-yOrig));
		if (x != xOrig)
			SetScrollPos(SB_HORZ, x);
		if (y != yOrig)
			SetScrollPos(SB_VERT, y);
	}
	return TRUE;
}

/////////////////////////////////////////////////////////////////////////////
// CLogScrollView Diagnose

#ifdef _DEBUG
void CLogScrollView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);

	dc << "m_totalLog = " << m_totalLog;
	dc << "\nm_totalDev = " << m_totalDev;
	dc << "\nm_pageDev = " << m_pageDev;
	dc << "\nm_lineDev = " << m_lineDev;
	dc << "\nm_bCenter = " << m_bCenter;
	dc << "\nm_bInsideUpdate = " << m_bInsideUpdate;
	dc << "\nm_nMapMode = ";
	dc << "MM_LOGICAL TWIPS";
	dc << "\n";
}
#endif //_DEBUG

#ifdef AFX_INIT_SEG
#pragma code_seg(AFX_INIT_SEG)
#endif

IMPLEMENT_DYNAMIC(CLogScrollView, CView)

/////////////////////////////////////////////////////////////////////////////
