// rowview.cpp : Implementierung der Klasse CRowView (MM_TEXT)
//

#include "stdafx.h"
#include "resource.h" // fr IDS_TOO_MANY_ROWS
#include "rowview.h"
#include <limits.h> // fr SHRT_MAX, LONG_MAX

//#define WINDOWSNT // fr unbegrenzte Zeilenanzahl

#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif

IMPLEMENT_DYNAMIC(CRowView, CScrollView)

/////////////////////////////////////////////////////////////////////////////
// CRowView

BEGIN_MESSAGE_MAP(CRowView, CScrollView)
    //{{AFX_MSG_MAP(CRowView)
    ON_WM_KEYDOWN()
    ON_WM_SIZE()
    ON_WM_LBUTTONDOWN()
    //}}AFX_MSG_MAP
    // Standard-Druckbefehle
    ON_COMMAND(ID_FILE_PRINT, CView::OnFilePrint)
    ON_COMMAND(ID_FILE_PRINT_PREVIEW, CView::OnFilePrintPreview)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CRowView Konstruktion, Initialisierung und Zerstrung
CRowView::CRowView()
{
    m_nPrevSelectedRow = 0;
}

CRowView::~CRowView()
{
}

/////////////////////////////////////////////////////////////////////////////
// CRowView Aktualisieren und Zeichnen

void CRowView::OnInitialUpdate()
{
    m_nPrevRowCount = GetRowCount();
    m_nPrevSelectedRow = GetActiveRow();
    OnUpdate(this, 0, NULL);
}

/////////////////////////////////////////////////////////////////////////////
void CRowView::UpdateRow(int nInvalidRow)
{
    int nRowCount = GetRowCount();
    // Falls sich die Zeilenzahl gendert hat, Bildlaufbereich anpassen.
    if (nRowCount != m_nPrevRowCount) {
        UpdateScrollSizes();
        m_nPrevRowCount = nRowCount;
    }

    // Wenn sich die aktuell ausgewhlte Zeile ndert,
    // die Ansichdt so verschieben, dass die ausgewhlte Zeile sichtbar
	// ist, und die abgeleitete Klasse zum Neuzeichnen der ausgewhlten
	// und der zuvor ausgewhlten Zeile auffordern.

    CClientDC dc(this);
    OnPrepareDC(&dc);

    // Bereich der gegenwrtig ganz sichtbaren Zeilen ermitteln.
	// Wir wollen die Anzeige diskret verschieben, so dass im Fenster 
	// stets auch die nchste oder vorherige Zeile ganz sichtbar ist.

    int   nFirstRow, nLastRow;
    CRect rectClient;

    GetClientRect(&rectClient);
    dc.DPtoLP(&rectClient);
    RectLPtoRowRange(rectClient, nFirstRow, nLastRow, FALSE);

    // Falls erforderlich, das Fenster so verschieben, dass die
	// neu ausgewhlte Zeile sichtbar ist.

    CPoint pt = GetDeviceScrollPosition();
    BOOL   bNeedToScroll = TRUE;

    if (nInvalidRow < nFirstRow) {
        // Die neu ausgewhlte Zeile befindet sich oberhalb der
		// aktuell sichtbaren Zeile. Die Ansicht soll so verschoben
		// werden, dass sich die neu ausgewhlte Zeile am oberen
		// Fensterrand befindet. Die letzte Zeile ist mglicherweise 
		// nicht ganz sichtbar. 
        pt.y = RowToYPos(nInvalidRow);
    }
    else if (nInvalidRow > nLastRow){
        // Die neu ausgewhlte Zeile befindet sich unterhalb der
		// aktuell sichtbaren Zeile. Die Ansicht soll so verschoben
		// werden, dass sich die neu ausgewhlte Zeile am unteren
		// Fensterrand befindet. Die erste Zeile ist mglicherweise 
		// nicht ganz sichtbar. 
        pt.y = max(0, RowToYPos(nInvalidRow+1) - rectClient.Height());
    }
    else {
        bNeedToScroll = FALSE;
    }
    if (bNeedToScroll) {
        ScrollToDevicePosition(pt);
        // Die neue ausgewhlte Zeile wird im ungltig gewordenen 
		// Fensterbereich neugezeichnet, falls ein Bildlauf durchgefhrt wird.

        OnPrepareDC(&dc);  // Der Gertekontext muss erneut eingestellt werden, 
            // da ScrollToDevicePosition() den Ursprung der Ansicht ndert.
            // Der Gertekontext wird weiter unten gebraucht.
    }

    CRect rectInvalid = RowToWndRect(&dc, nInvalidRow);
    InvalidateRect(&rectInvalid);

    // Die abgeleitete Klasse erhlt hier die Mglichkeit, die zuvor ausgewhlte
	// Zeile neu zu zeichnen und ggf. die Hervorhebung zu entfernen.

    int nSelectedRow = GetActiveRow();
    if (m_nPrevSelectedRow != nSelectedRow) {
        CRect rectOldSelection = RowToWndRect(&dc, m_nPrevSelectedRow);
        InvalidateRect(&rectOldSelection);
        m_nPrevSelectedRow = nSelectedRow;
    }
}

/////////////////////////////////////////////////////////////////////////////
void CRowView::UpdateScrollSizes()
{
    // UpdateScrollSizes() wird aufgerufen, wenn der Bildlaufbereich oder 
	// die Seiten-/Zeilengre angepasst werden mssen. Dies kann in 
	// zwei Fllen erforderlich sein: (1) wenn eine neue Zeile hinzugefgt
	// wird - siehe UpdateRow()- und (2) wenn sich die Fenstergre ndert
	// -siehe OnSize().

    CRect rectClient;
    GetClientRect(&rectClient);

    CClientDC dc(this);
    CalculateRowMetrics(&dc);

    // Der vertikale Bildlaufbereich erstreckt sich ber die Gesamtanzeigehhe 
    // aller Zeilen.
    CSize sizeTotal(m_nRowWidth, m_nRowHeight *
                   (min(GetRowCount(), LastViewableRow())));

    // Der vertikale Abstand fr das seitenweise Blttern entspricht der
    // Zeilenzahl, die im aktuellen Fenster angezeigt werden kann, minus 
    // einer Zeile (berlappung der Seitenanzeige).
    CSize sizePage(m_nRowWidth / 5, max(m_nRowHeight,
                ((rectClient.bottom / m_nRowHeight) - 1) * m_nRowHeight));

    // Der  vertikale Abstand fr das zeilenweise Blttern entspricht der
    // Zeilenhhe.
    CSize sizeLine(m_nRowWidth / 20, m_nRowHeight);

    SetScrollSizes(MM_TEXT, sizeTotal, sizePage, sizeLine);
}


/////////////////////////////////////////////////////////////////////////////
void CRowView::OnDraw(CDC* pDC)
{
    if (GetRowCount() == 0) {
        return;
    }
    // Das Fenster ist ungltig geworden und muss neu gezeichnet werden,
    // oder es muss eine Seite gedruckt oder in der Seitenansicht angezeigt werden.
    // Zuerst den Zeilenbereich ermitteln, der angezeigt oder 
	// gedruckt werden soll.

    int   nFirstRow, nLastRow;
    CRect rectClip;
    pDC->GetClipBox(&rectClip); // Ungltigen Bereich ermitteln.
    RectLPtoRowRange(rectClip, nFirstRow, nLastRow, TRUE);

    // Jede Zeile des ungltigen Fensterbereichs oder der gedruckten
    // Seite (oder Seitenansicht) zeichnen.

    int nActiveRow = GetActiveRow();
    int nRow, y;
    int nLastViewableRow = LastViewableRow();

    for (nRow = nFirstRow,  y = m_nRowHeight * nFirstRow;
            nRow <= nLastRow; nRow++, y += m_nRowHeight) {
        if (nRow >= (nLastViewableRow - 1)) {
            CString strWarning;
            strWarning.LoadString(IDS_TOO_MANY_ROWS);
            pDC->TextOut(0, y, strWarning);
            break;
        }
        // Abhngig davon, ob es sich um die aktuell ausgewhlte Zeile
		// handelt, Hervorhebung der Zeile vorbereiten bzw. Hervorhebung 
		// entfernen und Hintergrund (hinter dem Text) entsprechend
        // darstellen.
        CBrush brushBackground;
        COLORREF crOldText, crOldBackground;
        if (!pDC->IsPrinting()) {
            if (nRow == nActiveRow) {
                brushBackground.CreateSolidBrush(::GetSysColor(COLOR_HIGHLIGHT));
                crOldBackground = pDC->SetBkColor(::GetSysColor(COLOR_HIGHLIGHT));
                crOldText = pDC->SetTextColor(::GetSysColor(COLOR_HIGHLIGHTTEXT));
            }
            else {
                brushBackground.CreateSolidBrush(::GetSysColor(COLOR_WINDOW));
            }

            CRect rectSelection;
            pDC->GetClipBox(&rectSelection);
            rectSelection.top = y;
            rectSelection.bottom = y + m_nRowHeight;
            pDC->FillRect(&rectSelection, &brushBackground);
        }
        OnDrawRow(pDC, nRow, y, nRow == nActiveRow);
        // DC wiederherstellen. (VERLAGERT AUS ABGELEITETER KLASSE)
        if (!pDC->IsPrinting() && (nRow == nActiveRow)) {
            pDC->SetBkColor(crOldBackground);
            pDC->SetTextColor(crOldText);
        }
    }
}

/////////////////////////////////////////////////////////////////////////////
// Implementierung

int CRowView::RowToYPos(int nRow)
{
    return (nRow * m_nRowHeight);
}

/////////////////////////////////////////////////////////////////////////////
CRect CRowView::RowToWndRect(CDC* pDC, int nRow)
{
    CRect clientRect;

    GetClientRect(clientRect);
    pDC->DPtoLP(clientRect);
    CRect rect(clientRect.left, nRow * m_nRowHeight,
               clientRect.right, (nRow + 1) * m_nRowHeight);
    pDC->LPtoDP(&rect);
    return rect;
}

/////////////////////////////////////////////////////////////////////////////
int CRowView::LastViewableRow()
{
#ifdef WINDOWSNT
    return LONG_MAX / m_nRowHeight - 1;
#else
    return SHRT_MAX / m_nRowHeight - 1;
#endif
}

/////////////////////////////////////////////////////////////////////////////
void CRowView::RectLPtoRowRange(const CRect& rect, int& nFirstRow,
                                int& nLastRow,
                                BOOL bIncludePartiallyShownRows)
{
    int nRounding = bIncludePartiallyShownRows ? 0 : (m_nRowHeight - 1);
    nFirstRow = (rect.top + nRounding) / m_nRowHeight;
    nLastRow = min((rect.bottom - nRounding) / m_nRowHeight,
                   GetRowCount() - 1);
}

/////////////////////////////////////////////////////////////////////////////
void CRowView::OnPrepareDC(CDC* pDC, CPrintInfo* pInfo)
{
    // Die Gre des Texts, der angezeigt, ausgegeben oder vorangezeigt wird,
	// ndert sich abhngig vom Gertekontext. Wir rufen OnPrepareDC() auf,
	// um die CClientDC-Objekte, die zur Berechnung der Textpositionen
	// verwendet werden, und die Datenelemente des CRowView-Objekts 
	// vorzubereiten. Das Anwendungsgerst ruft OnPrepareDC() zudem auf, 
    // bevor der Gertekontext OnDraw() bergeben wird.

    CScrollView::OnPrepareDC(pDC, pInfo);
    CalculateRowMetrics(pDC);
}

/////////////////////////////////////////////////////////////////////////////
// berschreibungen von CView zur Implementierung der Druckfunktion

BOOL CRowView::OnPreparePrinting(CPrintInfo* pInfo)
{
    return DoPreparePrinting(pInfo);
}

/////////////////////////////////////////////////////////////////////////////
void CRowView::OnBeginPrinting(CDC* pDC, CPrintInfo* pInfo)
{
    // OnBeginPrinting() wird aufgerufen, nachdem der Anwender im Dialogfeld
	// 'Drucken' auf OK geklickt hat und nachdem das Anwendungsgerst ein
	// CDC-Objekt fr den Drucker oder die Seitenansicht erstellt hat.

    // Dies ist der geeignetet Zeitpunkt, den Seitenbereich festzulegen.
    // Anhand des CDC-Objekts knnen wir ermitteln, wie viele Zeilen auf
	// eine Seite passen und wie viele gedruckte Seiten das gesamte
	// Dokument ergibt.

    int nPageHeight = pDC->GetDeviceCaps(VERTRES);
    CalculateRowMetrics(pDC);
    m_nRowsPerPrintedPage = nPageHeight / m_nRowHeight;
    int nPrintableRowCount = LastViewableRow() + 1;
    if (GetRowCount() < nPrintableRowCount) {
        nPrintableRowCount = GetRowCount();
    }
    pInfo->SetMaxPage((nPrintableRowCount + m_nRowsPerPrintedPage - 1)
        / m_nRowsPerPrintedPage);
    pInfo->m_nCurPage = 1;  // Seitenansicht mit Seite #1 beginnen
}

/////////////////////////////////////////////////////////////////////////////
void CRowView::OnPrint(CDC* pDC, CPrintInfo* pInfo)
{
    // Zeilen der aktuellen Seite drucken

    int yTopOfPage = (pInfo->m_nCurPage -1) * m_nRowsPerPrintedPage
                     * m_nRowHeight;

    // Ansichtsbereich so ausrichdten, dass die erste gedruckte Zeile die
	// Koordinaten (0,0) hat.
    pDC->SetViewportOrg(0, -yTopOfPage);

    // So viele Zeilen zeichnen, wie auf die gedruckte Seite passen.
	// Gedruckte Seite so einrichten, dass die Zeile am Seitenende 
	// nicht abgeschnitten wird (diese Zeile wird am Anfang der nchsten
    // Seite ganz dargestellt).

    int nPageWidth = pDC->GetDeviceCaps(HORZRES);

    CRect rectClip = CRect(0, yTopOfPage, nPageWidth, 
         yTopOfPage + m_nRowsPerPrintedPage * m_nRowHeight);
    pDC->IntersectClipRect(&rectClip);
    OnDraw(pDC);
}

/////////////////////////////////////////////////////////////////////////////
// CRowView Befehle


void CRowView::OnSize(UINT nType, int cx, int cy)
{
    UpdateScrollSizes();
    CScrollView::OnSize(nType, cx, cy);
}

/////////////////////////////////////////////////////////////////////////////
void CRowView::OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags)
{
//  int nMin, nMax, nPos;
    CRect clientRect;

    GetClientRect(clientRect);
    CClientDC dc(this);
    dc.DPtoLP(clientRect);
    
    switch (nChar)
    {
        case VK_HOME:
            ChangeSelectionToRow(0);
            break;
        case VK_END:
            ChangeSelectionToRow(min(GetRowCount() - 1,
                                 LastViewableRow() - 2));
            break;
        case VK_UP:
            ChangeSelectionNextRow(FALSE);
            break;
        case VK_DOWN:
            ChangeSelectionNextRow(TRUE);
            break;
        case VK_LEFT:
            if (clientRect.Width() < GetTotalSize().cx) {
			      OnHScroll(SB_LINELEFT, 0, NULL);
            }
            break;
        case VK_RIGHT:
            if (clientRect.Width() < GetTotalSize().cx) {
			      OnHScroll(SB_LINERIGHT, 0, NULL);
            }
            break;
        case VK_PRIOR:
            if (clientRect.Height() < GetTotalSize().cy) {
			      OnVScroll(SB_PAGEUP, 0, NULL);
            }
            break;
        case VK_NEXT:
            if (clientRect.Height() < GetTotalSize().cy) {
			      OnVScroll(SB_PAGEDOWN, 0, NULL);
            }
            break;
        default:
            CScrollView::OnKeyDown(nChar, nRepCnt, nFlags);
    }
}

/////////////////////////////////////////////////////////////////////////////
void CRowView::OnLButtonDown(UINT, CPoint point)
{
    CClientDC dc(this);
    OnPrepareDC(&dc);
    dc.DPtoLP(&point);

    CRect rect(point, CSize(1,1));
    int   nFirstRow, nLastRow;

    RectLPtoRowRange(rect, nFirstRow, nLastRow, TRUE);
    if (nFirstRow <= (GetRowCount() - 1)) {
        ChangeSelectionToRow(nFirstRow);
    }
}

