// ex32avw.cpp : Implementierung der Klasse CEx32aView
//

#include "stdafx.h"
#include "ex32a.h"

#include "ex32adoc.h"
#include "ex32aView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CEx32aView

IMPLEMENT_DYNCREATE(CEx32aView, CRowView)

BEGIN_MESSAGE_MAP(CEx32aView, CRowView)
    //{{AFX_MSG_MAP(CEx32aView)
    ON_WM_SETFOCUS()
    ON_WM_MOUSEACTIVATE()
	ON_WM_TIMER()
	//}}AFX_MSG_MAP
    // Standard-Druckbefehle
    ON_COMMAND(ID_FILE_PRINT, CRowView::OnFilePrint)
    ON_COMMAND(ID_FILE_PRINT_PREVIEW, CRowView::OnFilePrintPreview)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CEx32aView Konstruktion/Destruktion

CEx32aView::CEx32aView()
{
	m_pSet = NULL;
}

CEx32aView::~CEx32aView()
{
}
									
/////////////////////////////////////////////////////////////////////////////
void CEx32aView::OnUpdate(CView*, LPARAM lHint, CObject* pHint)
{
    // Wird aufgerufen, wenn der Anwender eine neue Abfrage ausfhrt;
    // m_pSet kann nicht in OnInitialUpdate gesetzt werden, da dort NULL
    ScrollToPosition(CPoint(0, 0));
    Invalidate(TRUE);
    m_nSelectedRow = -1;
	CEx32aDoc* pDoc = GetDocument();
	if((m_pSet = pDoc->m_pRecordset) != NULL) {
		UpdateScrollSizes();
	}
}

/////////////////////////////////////////////////////////////////////////////
// CRowView-berschreibungen

/////////////////////////////////////////////////////////////////////////////
void CEx32aView::OnDrawRow(CDC* pDC, int nRow, int y, BOOL bSelected)
{
    int x = 0;
    int i;
    CEx32aDoc* pDoc = GetDocument();
    if (m_pSet == NULL) return;
	
    if (nRow == 0) {    // Titelzeile
      for (i = 0; i < pDoc->m_nFields; i++) {
        pDC->TextOut(x, y, pDoc->m_arrayFieldName[i]);
        x += pDoc->m_arrayFieldSize[i] * m_nCharWidth;
      }	
    }
    else {
      try {
        m_pSet->SetAbsolutePosition(nRow - 1); // anpassen fr Titelzeile
		// SetAbsolutePosition lst erst NACH Ende der Satzgruppe 
		//  eine Ausnahme aus
         if (m_pSet->GetAbsolutePosition() == (nRow - 1)) {
          DrawDataRow(pDC, y);
        }
      }
      catch (CDaoException* e) {
		// Lschung wird im Programm mglicherweise verzgert wiedergegeben
        if (e->m_pErrorInfo->m_lErrorCode == 3167) {
          pDC->TextOut(0, y, "**DATENSATZ GELSCHT**");
        }
        else {
          m_pSet->MoveLast(); // falls m_nRowCount zu gro ist
	      pDoc->m_nRowCount = m_pSet->GetAbsolutePosition() + 2;
        }
        e->Delete();
      }
    }
}

void CEx32aView::DrawDataRow(CDC* pDC, int y)
{
    int x = 0;
    CString strTime;

    COleVariant var;
    CString str;
    CEx32aDoc* pDoc = GetDocument();
    for (int i = 0; i < pDoc->m_nFields; i++) {
      var = m_pSet->GetFieldValue(i);
      switch (var.vt) {
      case VT_BSTR:
        str = (LPCSTR) var.bstrVal; // 1-Byte-Zeichen in DAO
        break;
      case VT_I2:
        str.Format("%d", (int) var.iVal);
        break;
      case VT_I4:
        str.Format("%d", var.lVal);
        break;
      case VT_R4:
        str.Format("%10.2f", (double) var.fltVal);
        break;
      case VT_R8:
        str.Format("%10.2f", var.dblVal);
        break;
	  case VT_CY:
        str = COleCurrency(var).Format();
        break;
      case VT_DATE:
        str = COleDateTime(var).Format();
        break;
      case VT_BOOL:
        str = (var.boolVal == 0) ? "FALSE" : "TRUE";
        break;
      case VT_NULL:
        str =  "----";
        break;
      default:
        str.Format("Unk type %d\n", var.vt);
        TRACE("Unknown type %d\n", var.vt);
      }
      pDC->TextOut(x, y, str);
      x += pDoc->m_arrayFieldSize[i] * m_nCharWidth;
    }	
}

/////////////////////////////////////////////////////////////////////////////
void CEx32aView::GetRowWidthHeight(CDC* pDC, int& nRowWidth,
         int& nRowHeight, int& nCharWidth)
{
    TEXTMETRIC tm;

    CEx32aDoc* pDoc = GetDocument();
	pDC->GetTextMetrics(&tm);
    nCharWidth = tm.tmAveCharWidth + 1;
    nRowWidth = 0;
	for(int i = 0; i < pDoc->m_nFields; i++) {
	    nRowWidth += pDoc->m_arrayFieldSize[i];
	}
    nRowWidth *= nCharWidth;
    nRowHeight = tm.tmHeight;
}

/////////////////////////////////////////////////////////////////////////////
int CEx32aView::GetActiveRow()
{
    return m_nSelectedRow;
}

/////////////////////////////////////////////////////////////////////////////
int CEx32aView::GetRowCount()
{
    return GetDocument()->m_nRowCount;
}

/////////////////////////////////////////////////////////////////////////////
void CEx32aView::ChangeSelectionNextRow(BOOL bNext)
{
    if (bNext && (m_nSelectedRow < min(GetRowCount() - 1,
                LastViewableRow() - 2))) {
        m_nSelectedRow++;
    }
    if (!bNext && m_nSelectedRow) {
        m_nSelectedRow--;
    }
    UpdateRow(m_nSelectedRow);
}

/////////////////////////////////////////////////////////////////////////////
void CEx32aView::ChangeSelectionToRow(int nRow)
{
    if ((nRow >= 0) && (nRow < min(GetRowCount(), LastViewableRow() - 1))) {
        m_nSelectedRow = nRow;
    }
    UpdateRow(m_nSelectedRow);
}

/////////////////////////////////////////////////////////////////////////////
// CEx32aView Diagnose

#ifdef _DEBUG
void CEx32aView::AssertValid() const
{
    CRowView::AssertValid();
}

void CEx32aView::Dump(CDumpContext& dc) const
{
    CRowView::Dump(dc);
}

/////////////////////////////////////////////////////////////////////////////
CEx32aDoc* CEx32aView::GetDocument() // Die endgltige (nicht zur Fehlersuche kompilierte) Version ist Inline
{
    ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CEx32aDoc)));
    return (CEx32aDoc*) m_pDocument;
}

#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CEx32aView Nachrichten-Handler

/////////////////////////////////////////////////////////////////////////////
void CEx32aView::OnSetFocus(CWnd* pOldWnd)
{
    // Abfrage-Zeichenfolge der Dialogleiste aktualisieren
	GetDocument()->PutQuery();
	CRowView::OnSetFocus(pOldWnd);
}

/////////////////////////////////////////////////////////////////////////////
int CEx32aView::OnMouseActivate(CWnd* pDesktopWnd, UINT nHitTest, UINT message)
{
	// Fokus von Dialogleiste neu setzen
    SetFocus(); 
    return CRowView::OnMouseActivate(pDesktopWnd, nHitTest, message);
}


void CEx32aView::OnInitialUpdate() 
{
	CRowView::OnInitialUpdate();
}

void CEx32aView::OnTimer(UINT nIDEvent) 
{
	Invalidate(); // Datenbankansicht aktualisieren
}
